<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Forecasting.php 4130 2006-01-26 18:05:33Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/max/Dal/MaintenanceCommon.php';
require_once MAX_PATH . '/lib/max/other/lib-limitations.inc.php';
require_once 'Date.php';

/**
 * The non-DB specific Data Access Layer (DAL) class for the Forecast Engine.
 *
 * Includes DAL methods for the Forecast Engine.
 *
 * @package    MaxDal
 * @author     Radek Maciaszek <radek@m3.net>
 */
class MAX_Dal_Forecasting extends MAX_Dal_Maintenance_Common
{
    var $start;
    var $end;
    var $operationIntervalStart;
    var $operationIntervalEnd;
    var $operationIntervalID;

    /**
     * The class constructor method.
     */
    function MAX_Dal_Forecasting()
    {
        parent::MAX_Dal_Maintenance_Common();
    }

    /**
     * A singleton method to obtain the same instance of the class.
     *
     * @access public
     */
    function &singleton()
    {
        static $instance;
        if (!is_a($instance, 'MAX_Dal_Forecasting')) {
            $instance = &new MAX_Dal_Forecasting();
        }
        return $instance;
    }

    /**
     * Set dates required to calculate interval and for sql limitations
     *
     * @param Date $start  Start date
     * @param Date $end    End date
     * @return boolean     True on success else false
     */
    function setIntervalDates($start, $end)
    {
        $this->start = $start;
        $this->end   = $end;

        $maintenance = &$this->_getMaintenance();

        // Check the start and end dates`
        if (!$maintenance->checkIntervalDates($start, $end, $this->conf['maintenance']['operationInterval'])) {
            MAX::raiseError('Bad interval dates: "'.$this->start->format('%Y-%m-%d %H:%M:%S').'", "'.
                $this->end->format('%Y-%m-%d %H:%M:%S').'"');
            return false;
        }

        // Get the start and end dates of the operation interval ID
        list($this->operationIntervalStart, $this->operationIntervalEnd) =
            $maintenance->convertDateToOperationIntervalStartAndEndDates($start);
        // Get the operation interval ID
        $this->operationIntervalID = $maintenance->convertDateToOperationIntervalID($this->operationIntervalStart);

        return true;
    }

    /**
     * Method return additional SQL containing time limitations
     *
     * @return array  Array containing sql limitations.
     *                Every item is an array of sql limitation and comparison (OR|AND)
     */
    function getIntervalLimitations()
    {
        $limitations = array();
        $limitations[] = array("date_time >= '".$this->start->format('%Y-%m-%d %H:%M:%S')."'", 'and');
        $limitations[] = array("date_time <= '".$this->end->format('%Y-%m-%d %H:%M:%S')."'",   'and');
        return $limitations;
    }

    /**
     * A method to store the last time when the maintenance priority script
     * ran.
     *
     * @param Date $start The time that the maintenance priority run started.
     * @param Date $end   The time that the maintenance priority run ended.
     * @param Date $updateTo The end of the last operation interval ID that
     *                       has been updated.
     */
    function setMaintenanceForecastingLastRunInfo($start, $end, $updateTo)
    {
        return parent::setScriptLastRunInfo($start, $end, $updateTo, 'log_maintenance_forecasting');
    }

    /**
     * A method to return the time that the maintenance forecasting scripts
     * were last run, and the operation interval that was in use at the time.
     *
     * @param string $rawTable  If run last info doesn't exists script method look into
     *                          raw table latest date_time
     * @return array An array containing the time that represents the last time
     *               maintenance forecasting was last run (started). Returns an
     *               array of 2 NULLs if maintenance forecasting has never been run.
     */
    function getMaintenanceForecastingLastRunInfo($rawTable = 'data_raw_ad_impression', $oCurrDate = null)
    {
        $dateTime = parent::getScriptLastRunInfo('log_maintenance_forecasting', array('operation_interval'), 'updated_to');
        if($dateTime) {
            return new Date($dateTime['updated_to']);
        } else {
            $dbh = &$this->_getDbConnection();
            // Maintenance statistics has never been run - query the raw data
            $tableName = $this->prefix . $this->conf['table'][$rawTable];
            if ($this->conf['table']['split']) {
                // get list of the split tables
                $tables = $dbh->getTables();
                foreach($tables as $table) {
                    $pos = strpos($table, $tableName);
                    if($pos === false) {
                        continue;
                    } else {
                        $tableName = $table;
                    }
                }

                // try today
                $date = new Date();
                $rawAdImpressionTable .= '_' . $date->format('%Y%m%d');
            }
            $query = "
                SELECT
                    raw.date_time AS date
                FROM
                    $tableName AS raw
                ORDER BY date ASC
                LIMIT 1";
            $result = $dbh->query($query);
            if (PEAR::isError($result) && !PEAR::isError($result, DB_ERROR_NOSUCHTABLE)) {
                MAX::raiseError($result, MAX_ERROR_DBFAILURE, PEAR_ERROR_DIE);
            }
            if ($result->numRows() > 0) {
                $row = $result->fetchRow();
                // Convert the impression time into the end of the previous operation interval or hour
                $date = &new Date($row['date']);
                if ($type == 0) {
                    list($date, $_) = Maintenance::convertDateToOperationIntervalStartAndEndDates($date);
                    $date->subtractSeconds(1);
                    return $date;
                } else {
                    $date = &new Date($date->format('%Y-%m-%d %H:00:00'));
                    $date->subtractSeconds(1);
                    return $date;
                }
            }
            // No raw data was found - use default one
            if($oCurrDate === null) {
                $oCurrDate = new Date();
            }
            return $oCurrDate;
        }
    }

    /**
     * A method to get all active agencies in the system.
     *
     * @return array  Array containing agencies ids
     */
    function getAgenciesIds()
    {
        $table = $this->prefix . $this->conf['table']['agency'];

        $this->queryBuilder->reset();
        $this->queryBuilder->setTable($table);
        $this->queryBuilder->setSelect('agencyid');
        $agencies = $this->queryBuilder->getAll();

        $agenciesIds = array();
        foreach ($agencies as $agency) {
            $agenciesIds[] = $agency['agencyid'];
        }
        return $agenciesIds;
    }

    /**
     * A method to get publishers IDs for agency
     *
     * @param int $agencyId  Agency ID
     * @return array  Array containing publishers ids
     */
    function getPublishersIds($agencyId)
    {
        $table = $this->prefix . $this->conf['table']['affiliates'];

        $this->queryBuilder->reset();
        $this->queryBuilder->setTable($table);
        $this->queryBuilder->setSelect('affiliateid');
        //$this->queryBuilder->addSelect('name');
        //$this->queryBuilder->addSelect('publiczones');
        $this->queryBuilder->addWhere('agencyid = '.$agencyId, 'AND');
        $publishers = $this->queryBuilder->getAll();

        $publisherIds = array();
        foreach ($publishers as $publisher) {
            $publisherIds[] = $publisher['affiliateid'];
        }
        return $publisherIds;
    }

    /**
     * A method to get zones IDs for publisher
     *
     * @param int $publisherId  Publisher ID
     * @return array  Array containing zones ids
     */
    function getZonesIds($publisherId)
    {
        $table = $this->prefix . $this->conf['table']['zones'];

        $this->queryBuilder->reset();
        $this->queryBuilder->setTable($table);
        $this->queryBuilder->setSelect('zoneid');
        //$this->queryBuilder->addSelect('zonename');
        $this->queryBuilder->addWhere('affiliateid = '.$publisherId, 'AND');
        $zones = $this->queryBuilder->getAll();

        $zoneIds = array();
        foreach ($zones as $zone) {
            $zoneIds[] = $zone['zoneid'];
        }
        return $zoneIds;
    }

    /**
     * Get list of channel ids for agency/publisher
     *
     * @param int $agencyId     Agency ID
     * @param int $publisherId  Publisher ID
     * @return array  Array containing channel IDs
     */
    function getChannelsIdsByAgencyPublisher($agencyId, $publisherId)
    {
        $table = $this->prefix . $this->conf['table']['channel'];

        $this->queryBuilder->reset();
        $this->queryBuilder->setTable($table);
        $this->queryBuilder->setSelect('channelid');
        //$this->queryBuilder->addSelect('channel_name');
        $this->queryBuilder->addWhere('agencyid = '.$agencyId, 'AND');
        $this->queryBuilder->addWhere('affiliateid = '.$publisherId, 'AND');
        $channels = $this->queryBuilder->getAll();

        $channelIds = array();
        foreach ($channels as $channel) {
            $channelIds[] = $channel['channelid'];
        }
        return $channelIds;
    }

    /**
     * Method return array of channel ids by agency ID
     *
     * @param int $agencyId  Agency ID
     *
     * @return array|boolean - Array containing zone ids (or false on error)
     */
    function getChannelsByAgencyId($agencyId)
    {
        return $this->getChannelsIdsByAgencyPublisher($agencyId, 0);
    }

    /**
     * This method create the query for channel forecasting using array with sql limitations and table name
     *
     * @param array $limitations  Array of limitations, every item is an array
     *                            which contain sql query and comparison
     * @param string $tableName   Table name
     * @param int $isCount        If true the query count number of records (use COUNT(*))
     * @return string  Created SQL query
     */
    function buildQueryFromLimitations($limitations, $tableName, $isCount = false)
    {
        if ($GLOBALS['_MAX']['CONF']['table']['split']) {
            $tableName = $tableName . '_' . $this->start->format('%Y%m%d');
        }
        $table = $this->prefix . $tableName;

        $limitations = array_merge($limitations, $this->getIntervalLimitations());
        $groupBy = '';

        if ($isCount) {
            $select = 'COUNT(*) as counter, zone_id';
            $groupBy = ' GROUP BY zone_id';
        } else {
            $select = '*';
        }

        $where = '';
        foreach ($limitations as $limitation) {
            list ($sql, $comparison) = $limitation;
            if (!empty($where)) {
                $where .= ' '.$comparison;
            }
            $where .= ' '.$sql;
        }
        $query = 'SELECT '.$select.' FROM '.$tableName.' WHERE '.$where.$groupBy;

        return $query;
    }

    /**
     * A method to count the number of records in zone using only SQL limitations
     *
     * @param array $aSqlLimitations  Array of limitations, each item is an array
     *                                which contains SQL query and comparison
     * @param string $tableName       Table name
     * @return array An array, each containing the number of records for each zone,
     *               indexed by zone ID.
     */
    function countNumberOfRecordsInZonesBySqlLimitations($aSqlLimitations, $tableName)
    {
        $query = $this->buildQueryFromLimitations($aSqlLimitations, $tableName, true);
        $this->dbh->expectError(DB_ERROR_NOSUCHTABLE);
        $result = &$this->dbh->query($query);
        if (PEAR::isError($result)) {
            $this->dbh->popExpect();
            if ($ret->code == DB_ERROR_NOSUCHTABLE) {
                return 0;
            }
            return false;
        }
        $this->dbh->popExpect();
        $counter = array();
        while ($row = &$result->fetchRow()) {
            $counter[$row['zone_id']] = $row['counter'];
        }
        return $counter;
    }

    /**
     * Count number of records in zone using sql and php limitations
     *
     * We have to include here some business logic
     *
     * @param array $aSqlLimitations  Array of SQL limitations
     * @param string $phpLimitations  "Compiled" PHP limitation
     * @param string $tableName       Raw table name where method will count records
     * @return int  Number of records which match requirements
     */
    function countNumberOfRecordsInZonesBySqlAndPhpLimitations($aSqlLimitations, $phpLimitations, $tableName)
    {
        $query = $this->buildQueryFromLimitations($aSqlLimitations, $tableName, false);

        $this->dbh->expectError(DB_ERROR_NOSUCHTABLE);
        $res =& $this->dbh->query($query);

        if(PEAR::isError($res)) {
            $this->dbh->popExpect();
            if($res->code == DB_ERROR_NOSUCHTABLE) {
                return 0;
            }
            $this->dbh->popExpect();
            return false;
        }

        $counter = array();
        // Check channel limitations against every record
        while ($row =& $res->fetchRow()) {
            // map raw data into $_SERVER values
            $GLOBALS['_MAX']['LIMITATION_DATA']['HTTP_USER_AGENT']      = $row['user_agent'];
            $GLOBALS['_MAX']['LIMITATION_DATA']['REMOTE_ADDR']          = $row['ip_address'];
            $GLOBALS['_MAX']['LIMITATION_DATA']['REMOTE_HOST']          = $row['host_name'];
            $GLOBALS['_MAX']['LIMITATION_DATA']['HTTP_ACCEPT_LANGUAGE'] = $row['language'];
            $GLOBALS['_MAX']['LIMITATION_DATA']['HTTP_REFERER']         = $row['referer'];
            $source = $row['channel'];

            $result = true;
            @eval('$result = ('.$phpLimitations.');');
            if($result) {
                @$counter[$row['zone_id']]++;
            }
        }
        return $counter;
    }

    /**
     * Returns an array of ad limitation hashes.
     *
        Array
        (
            [0] => Array
                (
                    [channelid] => 3
                    [logical] => and
                    [type] => date
                    [comparison] => !=
                    [data] => 2005-5-25
                    [executionorder] => 0
                )

            [1] => Array
         ...
     *
     * @param int $channelId  Channel ID
     * @return array
     */
    function getChannelLimitations($channelId, $fields = array(), $wheres = array(), $joins = array(), $orderBys = array())
    {
        $wheres = array();
        $wheres[] = array("channelid = $channelId", 'and');
        return parent::getDeliveryLimitations($fields, $wheres, $joins, $orderBys, 'acls_channel', 'channelid');
    }

    /**
     * Method save zones impression forecasts in stats_zone_impression_history table.
     *
     * @param int $channelId      Channel ID
     * @param array $zonesIds     Zones IDs
     * @param string $identifier  Identified - either: impressions, requests or clicks
     * @param int $count          Amount of data to save
     * @return boolean  True on success else false
     */
    function saveChannelSummaryForZones($channelId, $zonesIds, $identifier, $counter)
    {
        foreach ($zonesIds as $zoneId) {
            $count = isset($counter[$zoneId]) ? $counter[$zoneId] : 0;
            if($this->saveChannelSummary($channelId, $zoneId, $identifier, $count) === false) {
                return false;
            }
        }
        return true;
    }

    /**
     * A method to save zone impression forecasts to the stats_zone_impression_history table.
     *
     * @param int $channelId      Channel ID
     * @param int $zoneId         Zone ID
     * @param string $identifier  Identified - either: impressions, requests or clicks
     * @param int $count          Amount of data to save
     * @return boolean  True on success else false
     */
    function saveChannelSummary($channelId, $zoneId, $identifier, $count)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &$this->_getDbConnection();

        // Get operation interval libraray
        $oOperationInterval = OperationInterval::singleton();

        $dayHour = $this->start->format('%Y-%m-%d %H:00:00');
        $intervalId = $oOperationInterval->getIntervalID($this->start);
        $intervalStart = $this->start->format('%Y-%m-%d %H:%M:%S');
        $intervalEnd = $this->end->format('%Y-%m-%d %H:%M:%S');

        $count = (int) $count;


        $query = "
            SELECT COUNT(*) FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_channel_hourly']}
            WHERE
                channel_id = $channelId
                AND zone_id = $zoneId
                AND operation_interval_id = $intervalId
                AND interval_start = '$intervalStart'
                AND interval_end = '$intervalEnd'";
        $result = $dbh->getOne($query);
        if(PEAR::isError($result)) {
            return false;
        }

        if($result) {
            // Update the existing summary
            $query = "
                UPDATE
                    {$conf['table']['prefix']}{$conf['table']['data_summary_channel_hourly']}
                SET
                    $identifier = $count
                WHERE
                    channel_id = $channelId
                    AND zone_id = $zoneId
                    AND operation_interval_id = $intervalId
                    AND interval_start = '$intervalStart'
                    AND interval_end = '$intervalEnd'";
            $result = $dbh->query($query);
            if(PEAR::isError($result)) {
                return false;
            }
        } else {
            // If none records were updated insert as a new summary
            $query = "
                INSERT INTO
                    {$conf['table']['prefix']}{$conf['table']['data_summary_channel_hourly']}
                    (
                        dayhour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        channel_id,
                        zone_id,
                        $identifier
                    )
                VALUES
                    (
                        '$dayHour',
                        {$oOperationInterval->interval},
                        $intervalId,
                        '$intervalStart',
                        '$intervalEnd',
                        $channelId,
                        $zoneId,
                        $count
                    )";
            $result = $dbh->query($query);
            if(PEAR::isError($result)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Get summary for channel/zone for a period of time
     *
     * @param int $channelId     Channel ID
     * @param int $zoneId        Zone ID
     * @param string $startDare  Start date
     * @param string $endDate    End day
     *
     * @return mixed  Return summary array or false on any error
     */
    function getChannelSummary($channelId, $zoneId, $startDate, $endDate)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &$this->_getDbConnection();

        $query = "
            SELECT * FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_channel_hourly']}
            WHERE
                channel_id = $channelId
                AND zone_id = $zoneId
                AND dayhour >= '$startDate'
                AND dayhour <= '$endDate'";

        $result = $dbh->query($query);
        if (PEAR::isError($result)) {
            MAX::raiseError($result, MAX_ERROR_DBFAILURE);
            return false;
        }
        $aSummary = array();
        while ($row = $result->fetchRow()) {
            $aSummary[] = $row;
        }

        return $aSummary;
    }

    /**
     * Get summary for channel/zone for a period of time
     *
     * @param int $channelId     Channel ID
     * @param int $zoneId        Zone ID
     * @param string $startDate  Start date
     * @param string $endDate    End date
     *
     * @return mixed  Return summary array or false on any error
     */
    function getChannelSummaryForZones($channelId, $zonesIds, $startDate, $endDate)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &$this->_getDbConnection();

        $query = "
            SELECT
                dayhour,
                sum(impressions) AS impressions,
                sum(clicks) AS clicks,
                sum(requests) AS requests
            FROM
                {$conf['table']['prefix']}{$conf['table']['data_summary_channel_hourly']}
            WHERE
                channel_id = $channelId
                AND (zone_id IN (".implode(',', $zonesIds)."))
                AND dayhour >= '$startDate'
                AND dayhour <= '$endDate'
            GROUP BY dayhour
            ORDER BY dayhour";

        $result = $dbh->query($query);
        if (PEAR::isError($result)) {
            MAX::raiseError($result, MAX_ERROR_DBFAILURE);
            return false;
        }
        $aSummary = array();
        while ($row = $result->fetchRow()) {
            $aSummary[$row['dayhour']] = $row;
        }

        return $aSummary;
    }

    /**
     * Obtains a database-level lock for the Maintenance Priority Engine process.
     *
     * @return boolean True if lock was obtained, false otherwise.
     */
    function obtainForecastingLock()
    {
        return parent::obtainLock('forecasting');
    }

    /**
     * Releases the database-level lock for the Maintenance Priority Engine process.
     *
     * @return mixed True if lock was released, a PEAR Error otherwise.
     */
    function releaseForecastingLock()
    {
        return parent::releaseLock('forecasting');
    }

    /**
     * Retrieve zones linked to ad.
     *
     * @param int $adId  Ad ID (banner ID)
     * @return array  Array of linked zones IDs
     */
    function getPubslihersAndZonesIdsByAdId($adId)
    {
        static $ids;
        static $staticAdId;
        if(isset($staticAdId) && $staticAdId == $adId) {
            return $ids;
        }
        $staticAdId = $adId;

        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &$this->_getDbConnection();

        $adId = (int) $adId;

        // Get list of all publishers connnected to chosen banner
        $query = "SELECT az.zone_id, z.affiliateid FROM
					".$conf['table']['prefix'].$conf['table']['ad_zone_assoc']." AS az,
					".$conf['table']['prefix'].$conf['table']['zones']." AS z
			    WHERE
					az.zone_id = z.zoneid
					AND az.ad_id = ".$adId;

        $result = $dbh->query($query);
        if (PEAR::isError($result)) {
            MAX::raiseError($result, MAX_ERROR_DBFAILURE);
            return false;
        }

        $ids = array();
        $ids['publishers'] = array();
        $ids['zones'] = array();

        while ($row = $result->fetchRow()) {
            $ids['publishers'][$row['affiliateid']] = $row['affiliateid'];
            $ids['zones'][$row['zone_id']]          = $row['zone_id'];
        }

        return $ids;
    }

    /**
     * Retrieve publishers linked to ad.
     *
     * @param int $adId  Ad ID (banner ID)
     * @return array  Array of linked publisher's IDs
     */
    function getPublishersIdsByAdId($adId)
    {
        $ids = $this->getPubslihersAndZonesIdsByAdId($adId);
        if($ids === false) {
            return false;
        }
        return $ids['publishers'];
    }

    /**
     * Retrieve zones IDs linked to ad.
     *
     * @param int $adId  Ad ID (banner ID)
     * @return array  Array of linked zones' IDs
     */
    function getZonesIdsByAdId($adId)
    {
        $ids = $this->getPubslihersAndZonesIdsByAdId($adId);
        if($ids === false) {
            return false;
        }
        return $ids['zones'];
    }

    /**
     * Retrieve publishers linked to ad.
     *
     * @param int $adId  Ad ID (banner ID)
     * @return array  Array of linked publisher's IDs
     */
    function getCampaignByAdId($adId)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &$this->_getDbConnection();

        $adId = (int) $adId;

        // Get list of all connnected publishers to chosen banner
        $query = "SELECT c.* FROM
					".$conf['table']['prefix'].$conf['table']['banners']." AS b,
					".$conf['table']['prefix'].$conf['table']['campaigns']." AS c
			    WHERE
					c.campaignid = b.campaignid
					AND b.bannerid = ".$adId;

        $result = $dbh->query($query);
        if (PEAR::isError($result)) {
            MAX::raiseError($result, MAX_ERROR_DBFAILURE);
            return false;
        }
        $row = $result->fetchRow();

        return $row;
    }

    /**
     * Retrieve channels added as a limitations to ad
     *
     * @param int $adId  Ad ID (banner ID)
     * @return array  Array of linked channel's IDs
     */
    function getAdChannelsIds($adId)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &$this->_getDbConnection();

        $adId = (int) $adId;

        // Get list of all linked channels to chosen banner
        $query = "SELECT data FROM
					".$conf['table']['prefix'].$conf['table']['acls']."
			    WHERE
					bannerid = ".$adId."
					AND type = 'channel'
				ORDER BY data";

        $result = $dbh->query($query);
        if (PEAR::isError($result)) {
            MAX::raiseError($result, MAX_ERROR_DBFAILURE);
            return false;
        }
        $channelsIds = array();
        while ($row = $result->fetchRow()) {
            $channelsIds[] = (int) $row['data'];
        }

        return $channelsIds;
    }
    /**
     * Retrieve channels ids available for agency and affiliates - channels which could be linked to
     * ad in ad's delivery limitations page.
     *
     * @param int $agencyId        Agency ID
     * @param array $affiliateIds  Affiliates IDs
     * @return array  Array of channel ids
     */
    function getAvailableChannelsByAgencyAndAffiliates($agencyId, $affiliatesIds)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $dbh = &$this->_getDbConnection();

        $affiliatesWhere = '';
        if(!empty($affiliatesIds)) {
            $affiliatesWhere = 'OR affiliateid IN ('.implode(',', $affiliatesIds).'))';
        }

        // Get list of all linked channels to chosen banner
        $query = "SELECT channelid FROM
					".$conf['table']['prefix'].$conf['table']['channel']."
			    WHERE
					agencyid = ".$agencyId."
					AND (affiliateid=0 $affiliatesWhere
				ORDER BY name";

        $result = $dbh->query($query);
        if (PEAR::isError($result)) {
            MAX::raiseError($result, MAX_ERROR_DBFAILURE);
            return false;
        }
        $channelsIds = array();
        while ($row = $result->fetchRow()) {
            $channelsIds[] = $row['channelid'];
        }

        return $channelsIds;
    }
}

?>