<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: DeliveryLimitation.php 4183 2006-02-09 10:10:13Z andrew@m3.net $
*/

require_once 'Date.php';

/**
 * An abstract class that is used to store and manipulate individual delivery
 * limitations for ads, with the goal of determining when (if at all) the
 * deliver limitation "blocks" (as oppsed to "filters") deliver of an
 * advertisement.
 *
 * @package    MaxMaintenance
 * @subpackage Priority
 * @author     Andrew Hill <andrew@m3.net>
 * @author     James Floyd <james@m3.net>
 *
 * @abstract
 */
class MAX_Maintenance_Priority_DeliveryLimitation
{
    /**
     * Logical operator: and, or
     * @var string
     */
    var $logical;

    /**
     * Delivery limitation type
     * @var string
     */
    var $type;

    /**
     * Delivery limitation comparison: ==, !=, >=, <=, >, <
     * @var string
     */
    var $comparison;

    /**
     * Delivery limitation data
     * @var array
     */
    var $data = array();

    /**
     * Order delivery limitation should be executed in: 0-n
     * @var integer
     */
    var $executionOrder;

    /**
     * Constructor method.
     *
     * @param array $aDeliveryLimitation An array containing the details of a delivery limitation
     *                                   associated with an ad. For example:
     *                                   array(
     *                                       [ad_id]             => 1
     *                                       [logical]           => and
     *                                       [type]              => Time:Hour
     *                                       [comparison]        => ==
     *                                       [data]              => 1,7,18,23
     *                                       [executionorder]    => 1
     *                                   )
     * @return MAX_Maintenance_Priority_DeliveryLimitation
     */
    function MAX_Maintenance_Priority_DeliveryLimitation($aDeliveryLimitation)
    {
        // Store the logical, type, comparison, data and execution order
        // items of the delivery limitation
        $this->logical        = $aDeliveryLimitation['logical'];
        $this->type           = $aDeliveryLimitation['type'];
        $this->comparison     = $aDeliveryLimitation['comparison'];
        $this->data           = explode(',',$aDeliveryLimitation['data']);
        $this->executionOrder = $aDeliveryLimitation['executionorder'];
        // Manipulate all delivery limitations to be expressed in terms of when
        // NOT to deliver the ad, as opposed to when to deliver
        $this->calculateNonDeliveryDeliveryLimitation();
    }

    /**
     * A factory method to return the appropriate MAX_Maintenance_Priority_DeliveryLimitation
     * subclass object (one of MAX_Maintenance_Priority_DeliveryLimitation_Date,
     * MAX_Maintenance_Priority_DeliveryLimitation_Day,
     * MAX_Maintenance_Priority_DeliveryLimitation_Empty or
     * MAX_Maintenance_Priority_DeliveryLimitation_Hour), depending on the data provided.
     *
     * @static
     * @param array $aDeliveryLimitation An array containing the details of a delivery limitation
     *                                   associated with an ad. For example:
     *                                   array(
     *                                       [ad_id]             => 1
     *                                       [logical]           => and
     *                                       [type]              => Time:Hour
     *                                       [comparison]        => ==
     *                                       [data]              => 1,7,18,23
     *                                       [executionorder]    => 1
     *                                   )
     * @return object MAX_Maintenance_Priority_DeliveryLimitation
     */
    function &factory($aDeliveryLimitation)
    {
        // Define an array naming the 3 date/time ACLs associated with the
        // MAX_Maintenance_Priority_DeliveryLimitation_Date,
        // MAX_Maintenance_Priority_DeliveryLimitation_Day
        // and MAX_Maintenance_Priority_DeliveryLimitation_Hour classes
        $dateTimeClasses = array('time:date', 'time:day', 'time:hour');
        // If the ACL properties passed in have a type that matches one of
        // the date/time ACLs, set the $class variable so that the appropriate
        // class can be instantiated, otherwise, set $class so that the
        // MAX_Maintenance_Priority_DeliveryLimitation_Empty class can be
        // instantiated
        if (in_array(strtolower($aDeliveryLimitation['type']), $dateTimeClasses)) {
            $class = ucfirst(substr($aDeliveryLimitation['type'], 5));
        } else {
            $class = 'Empty';
        }
        // Prepare the MAX_Maintenance_Priority_DeliveryLimitation subclass name
        $className = 'MAX_Maintenance_Priority_DeliveryLimitation_' . $class;
        // Instantiate the appropriate delivery limitation class
        $file = MAX_PATH . '/lib/max/Maintenance/Priority/DeliveryLimitation/' . $class . '.php';
        if (file_exists($file)) {
            include_once $file;
            if (class_exists($className)) {
                return new $className($aDeliveryLimitation);
            }
        }
    }

    /**
     * A method to convert delivery limitations into negative form (i.e. when
     * NOT to deliver ad, as opposed to when to deliver).
     *
     * @abstract
     * @return mixed Void, or a PEAR::Error.
     */
    function calculateNonDeliveryDeliveryLimitation() {
        return MAX::raiseError(
            'MAX_Maintenance_Priority_DeliveryLimitation::calculateNonDeliveryDeliveryLimitation() is abstract!',
            MAX_ERROR_NOMETHOD
        );
    }

    /**
     * A method to return the number of minutes each delivery limitation covers.
     *
     * @abstract
     * @return mixed An integer, giving the number of minutes the limitation covers,
     *               or a PEAR::Error.
     */
    function minutesPerTimePeriod()
    {
        return MAX::raiseError(
            'MAX_Maintenance_Priority_DeliveryLimitation::minutesPerTimePeriod() is abstract!',
            MAX_ERROR_NOMETHOD
        );
    }

    /**
     * A method to determine if the delivery limitation stored will prevent an
     * ad from delivering or not, given a time/date.
     *
     * @abstract
     * @param object $oDate PEAR:Date, represeting the time/date to test if the ACL would
     *                      block delivery at that point in time.
     * @return mixed A boolean (true if the ad is BLOCKED (i.e. will NOT deliver), false
     *               if the ad is NOT BLOCKED (i.e. WILL deliver), or a PEAR::Error.
     */
    function deliveryBlocked($oDate) {
        return MAX::raiseError(
            'MAX_Maintenance_Priority_DeliveryLimitation::deliveryBlocked() is abstract!',
            MAX_ERROR_NOMETHOD
        );
    }

    /**
     * A private method to return the appropriate operator to use when
     * evaulating delivery limitations in terms of when NOT to deliver as
     * opposed to when to deliver.
     *
     * For example, the delivery condition to deliver on a specific date
     * would be "Time:Date == 20050403". The same condition in terms of when
     * not to deliver will be "Time:Date != 20050403"; that is, the operator
     * changes from "==" to "!=".
     *
     * @access private
     * @param string $operator Operator, one of: ==, !=, >, >=, <, <=
     * @return mixed The appropriate (changed) delivery operator, or a
     *               PEAR::error object
     */
    function _getNonDeliveryOperator($operator)
    {
        if ($operator == '!=') {
            return '==';
        } elseif ($operator == '==') {
            return '!=';
        } elseif ($operator == '>') {
            return '<=';
        } elseif ($operator == '>=') {
            return '<';
        } elseif ($operator == '<') {
            return '>=';
        } elseif ($operator == '<=') {
            return '>';
        }
        return MAX::raiseError(
            'Illegal operator passed to ' . __CLASS__ . '::' . __FUNCTION__ ,
            MAX_ERROR_INVALIDARGS
        );
    }

}

?>
